SEXP_CONV_PATH "Goba"

open VecOps

type init = { init_at : float * float;
	      init_time_left : float }
with sexp

let explosion_radius = 200.0
let explosion_time = 5.0
let explosion_interval = 1.0 /. 20.0

class t id init0 =
  let init = Object.default_with init0 init_of_sexp in

object (self)
  inherit Object.t id as super

  val mutable alive = true
  val mutable explosion_due = 0.0
  val mutable time_left = init.init_time_left

  method set_location _ = ()
  method set_speed _ = ()

  method get_type = ObjectType.Bomb

  method collision_area =
    ((0.0, 0.0), (0.0, 0.0)), CollisionMask.empty_mask

  method is_alive = alive

  method location = init.init_at
  method speed = 0.0, 0.0

  method get_state delay =
    Object.sexp_of_state sexp_of_init
      { Object.state_super = super#get_state delay;
	state_init = { init_at = init.init_at;
		       init_time_left = time_left } }

  method set_state state delay =
    let state = Object.state_of_sexp init_of_sexp state in
      super#set_state state.Object.state_super delay;
      let init = state.Object.state_init in
	time_left <- init.init_time_left

  method tick delta world =
    if alive && self#self_owned then
      begin
	self#change;
	time_left <- time_left -. delta;
	if time_left < 0.0 then
	  alive <- false
	else
	  begin
	    explosion_due <- explosion_due -. delta;
	    if explosion_due < 0.0 then
	      begin
		explosion_due <- explosion_due +. explosion_interval;
		let explosion_ofs =
		  let angle = Random.float GobaMath.pi2 in
		  let dist = explosion_radius *. (Random.float 1.0 ** 1.5) in
		    Vector.v2 dist *|. (cos angle, sin angle)
		in
		  world#create_object 
		    (fun id ->
		       (new Explosion.t id (Object.Init 
					      { Explosion.init_at = init.init_at +|. explosion_ofs;
						init_burst = (0.0, 0.0);
						init_sound = true;
						init_damage = (100.0, 10.0, 20.0, 1.5) }) :> Object.base))
	      end
	  end
      end

  method render gfx flags at = ()

  initializer
    self#change;
end
